@@ -1,5 +1,7 @@
 -- Minetest: builtin/item.lua
 
+local builtin_shared = ...
+
 local function copy_pointed_thing(pointed_thing)
 	return {
 		type  = pointed_thing.type,
@@ -107,7 +109,7 @@ local facedir_to_dir_map = {
 	1, 4, 3, 2,
 }
 function core.facedir_to_dir(facedir)
-	return facedir_to_dir[facedir_to_dir_map[facedir]]
+	return facedir_to_dir[facedir_to_dir_map[facedir % 32]]
 end
 
 function core.dir_to_wallmounted(dir)
@@ -142,13 +144,56 @@ local wallmounted_to_dir = {
 	{x =  0, y =  0, z = -1},
 }
 function core.wallmounted_to_dir(wallmounted)
-	return wallmounted_to_dir[wallmounted]
+	return wallmounted_to_dir[wallmounted % 8]
 end
 
-function core.get_node_drops(nodename, toolname)
-	local drop = ItemStack({name=nodename}):get_definition().drop
+function core.dir_to_yaw(dir)
+	return -math.atan2(dir.x, dir.z)
+end
+
+function core.yaw_to_dir(yaw)
+	return {x = -math.sin(yaw), y = 0, z = math.cos(yaw)}
+end
+
+function core.is_colored_paramtype(ptype)
+	return (ptype == "color") or (ptype == "colorfacedir") or
+		(ptype == "colorwallmounted")
+end
+
+function core.strip_param2_color(param2, paramtype2)
+	if not core.is_colored_paramtype(paramtype2) then
+		return nil
+	end
+	if paramtype2 == "colorfacedir" then
+		param2 = math.floor(param2 / 32) * 32
+	elseif paramtype2 == "colorwallmounted" then
+		param2 = math.floor(param2 / 8) * 8
+	end
+	-- paramtype2 == "color" requires no modification.
+	return param2
+end
+
+function core.get_node_drops(node, toolname)
+	-- Compatibility, if node is string
+	local nodename = node
+	local param2 = 0
+	-- New format, if node is table
+	if (type(node) == "table") then
+		nodename = node.name
+		param2 = node.param2
+	end
+	local def = core.registered_nodes[nodename]
+	local drop = def and def.drop
+	local ptype = def and def.paramtype2
+	-- get color, if there is color (otherwise nil)
+	local palette_index = core.strip_param2_color(param2, ptype)
 	if drop == nil then
 		-- default drop
+		if palette_index then
+			local stack = ItemStack(nodename)
+			stack:get_meta():set_int("palette_index", palette_index)
+			return {stack:to_string()}
+		end
 		return {nodename}
 	elseif type(drop) == "string" then
 		-- itemstring drop
@@ -170,6 +215,8 @@ function core.get_node_drops(nodename, toolname)
 		end
 		if item.tools ~= nil then
 			good_tool = false
+		end
+		if item.tools ~= nil and toolname then
 			for _, tool in ipairs(item.tools) do
 				if tool:sub(1, 1) == '~' then
 					good_tool = toolname:find(tool:sub(2)) ~= nil
@@ -180,10 +227,16 @@ function core.get_node_drops(nodename, toolname)
 					break
 				end
 			end
-        	end
+		end
 		if good_rarity and good_tool then
 			got_count = got_count + 1
 			for _, add_item in ipairs(item.items) do
+				-- add color, if necessary
+				if item.inherit_color and palette_index then
+					local stack = ItemStack(add_item)
+					stack:get_meta():set_int("palette_index", palette_index)
+					add_item = stack:to_string()
+				end
 				got_items[#got_items+1] = add_item
 			end
 			if drop.max_items ~= nil and got_count == drop.max_items then
@@ -194,8 +247,22 @@ function core.get_node_drops(nodename, toolname)
 	return got_items
 end
 
-function core.item_place_node(itemstack, placer, pointed_thing, param2)
-	local item = itemstack:peek_item()
+local function user_name(user)
+	return user and user:get_player_name() or ""
+end
+
+local function is_protected(pos, name)
+	return core.is_protected(pos, name) and
+		not minetest.check_player_privs(name, "protection_bypass")
+end
+
+-- Returns a logging function. For empty names, does not log.
+local function make_log(name)
+	return name ~= "" and core.log or function() end
+end
+
+function core.item_place_node(itemstack, placer, pointed_thing, param2,
+		prevent_after_place)
 	local def = itemstack:get_definition()
 	if def.type ~= "node" or pointed_thing.type ~= "node" then
 		return itemstack, false
@@ -205,20 +272,22 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 	local oldnode_under = core.get_node_or_nil(under)
 	local above = pointed_thing.above
 	local oldnode_above = core.get_node_or_nil(above)
+	local playername = user_name(placer)
+	local log = make_log(playername)
 
 	if not oldnode_under or not oldnode_above then
-		core.log("info", placer:get_player_name() .. " tried to place"
+		log("info", playername .. " tried to place"
 			.. " node in unloaded position " .. core.pos_to_string(above))
 		return itemstack, false
 	end
 
-	local olddef_under = ItemStack({name=oldnode_under.name}):get_definition()
+	local olddef_under = core.registered_nodes[oldnode_under.name]
 	olddef_under = olddef_under or core.nodedef_default
-	local olddef_above = ItemStack({name=oldnode_above.name}):get_definition()
+	local olddef_above = core.registered_nodes[oldnode_above.name]
 	olddef_above = olddef_above or core.nodedef_default
 
 	if not olddef_above.buildable_to and not olddef_under.buildable_to then
-		core.log("info", placer:get_player_name() .. " tried to place"
+		log("info", playername .. " tried to place"
 			.. " node in invalid position " .. core.pos_to_string(above)
 			.. ", replacing " .. oldnode_above.name)
 		return itemstack, false
@@ -229,28 +298,30 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 
 	-- If node under is buildable_to, place into it instead (eg. snow)
 	if olddef_under.buildable_to then
-		core.log("info", "node under is buildable to")
+		log("info", "node under is buildable to")
 		place_to = {x = under.x, y = under.y, z = under.z}
 	end
 
-	if core.is_protected(place_to, placer:get_player_name()) and
-			not minetest.check_player_privs(placer, "protection_bypass") then
-		core.log("action", placer:get_player_name()
+	if is_protected(place_to, playername) then
+		log("action", playername
 				.. " tried to place " .. def.name
 				.. " at protected position "
 				.. core.pos_to_string(place_to))
-		core.record_protection_violation(place_to, placer:get_player_name())
+		core.record_protection_violation(place_to, playername)
 		return itemstack
 	end
 
-	core.log("action", placer:get_player_name() .. " places node "
+	log("action", playername .. " places node "
 		.. def.name .. " at " .. core.pos_to_string(place_to))
 
 	local oldnode = core.get_node(place_to)
-	local newnode = {name = def.name, param1 = 0, param2 = param2}
+	local newnode = {name = def.name, param1 = 0, param2 = param2 or 0}
 
 	-- Calculate direction for wall mounted stuff like torches and signs
-	if def.paramtype2 == 'wallmounted' and not param2 then
+	if def.place_param2 ~= nil then
+		newnode.param2 = def.place_param2
+	elseif (def.paramtype2 == "wallmounted" or
+			def.paramtype2 == "colorwallmounted") and not param2 then
 		local dir = {
 			x = under.x - above.x,
 			y = under.y - above.y,
@@ -258,8 +329,9 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 		}
 		newnode.param2 = core.dir_to_wallmounted(dir)
 	-- Calculate the direction for furnaces and chests and stuff
-	elseif def.paramtype2 == 'facedir' and not param2 then
-		local placer_pos = placer:getpos()
+	elseif (def.paramtype2 == "facedir" or
+			def.paramtype2 == "colorfacedir") and not param2 then
+		local placer_pos = placer and placer:getpos()
 		if placer_pos then
 			local dir = {
 				x = above.x - placer_pos.x,
@@ -267,14 +339,33 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 				z = above.z - placer_pos.z
 			}
 			newnode.param2 = core.dir_to_facedir(dir)
-			core.log("action", "facedir: " .. newnode.param2)
+			log("action", "facedir: " .. newnode.param2)
+		end
+	end
+
+	local metatable = itemstack:get_meta():to_table().fields
+
+	-- Transfer color information
+	if metatable.palette_index and not def.place_param2 then
+		local color_divisor = nil
+		if def.paramtype2 == "color" then
+			color_divisor = 1
+		elseif def.paramtype2 == "colorwallmounted" then
+			color_divisor = 8
+		elseif def.paramtype2 == "colorfacedir" then
+			color_divisor = 32
+		end
+		if color_divisor then
+			local color = math.floor(metatable.palette_index / color_divisor)
+			local other = newnode.param2 % color_divisor
+			newnode.param2 = color * color_divisor + other
 		end
 	end
 
 	-- Check if the node is attached and if it can be placed there
 	if core.get_item_group(def.name, "attached_node") ~= 0 and
-		not check_attached_node(place_to, newnode) then
-		core.log("action", "attached node " .. def.name ..
+		not builtin_shared.check_attached_node(place_to, newnode) then
+		log("action", "attached node " .. def.name ..
 			" can not be placed at " .. core.pos_to_string(place_to))
 		return itemstack, false
 	end
@@ -285,7 +376,7 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 	local take_item = true
 
 	-- Run callback
-	if def.after_place_node then
+	if def.after_place_node and not prevent_after_place then
 		-- Deepcopy place_to and pointed_thing because callback can modify it
 		local place_to_copy = {x=place_to.x, y=place_to.y, z=place_to.z}
 		local pointed_thing_copy = copy_pointed_thing(pointed_thing)
@@ -296,7 +387,6 @@ function core.item_place_node(itemstack, placer, pointed_thing, param2)
 	end
 
 	-- Run script hook
-	local _, callback
 	for _, callback in ipairs(core.registered_on_placenodes) do
 		-- Deepcopy pos, node and pointed_thing because callback can modify them
 		local place_to_copy = {x=place_to.x, y=place_to.y, z=place_to.z}
@@ -346,28 +436,27 @@ function core.item_secondary_use(itemstack, placer)
 end
 
 function core.item_drop(itemstack, dropper, pos)
-	if dropper and dropper:is_player() then
-		local v = dropper:get_look_dir()
-		local p = {x=pos.x, y=pos.y+1.2, z=pos.z}
-		local cs = itemstack:get_count()
+	local dropper_is_player = dropper and dropper:is_player()
+	local p = table.copy(pos)
+	local cnt = itemstack:get_count()
+	if dropper_is_player then
+		p.y = p.y + 1.2
 		if dropper:get_player_control().sneak then
-			cs = 1
+			cnt = 1
 		end
-		local item = itemstack:take_item(cs)
-		local obj = core.add_item(p, item)
-		if obj then
-			v.x = v.x*2
-			v.y = v.y*2 + 2
-			v.z = v.z*2
-			obj:setvelocity(v)
+	end
+	local item = itemstack:take_item(cnt)
+	local obj = core.add_item(p, item)
+	if obj then
+		if dropper_is_player then
+			local dir = dropper:get_look_dir()
+			dir.x = dir.x * 2.9
+			dir.y = dir.y * 2.9 + 2
+			dir.z = dir.z * 2.9
+			obj:set_velocity(dir)
 			obj:get_luaentity().dropped_by = dropper:get_player_name()
-			return itemstack
-		end
-
-	else
-		if core.add_item(pos, itemstack) then
-			return itemstack
 		end
+		return itemstack
 	end
 	-- If we reach this, adding the object to the
 	-- environment failed
@@ -388,7 +477,8 @@ function core.do_item_eat(hp_change, replace_with_item, itemstack, user, pointed
 				itemstack:add_item(replace_with_item)
 			else
 				local inv = user:get_inventory()
-				if inv:room_for_item("main", {name=replace_with_item}) then
+				-- Check if inv is null, since non-players don't have one
+				if inv and inv:room_for_item("main", {name=replace_with_item}) then
 					inv:add_item("main", replace_with_item)
 				else
 					local pos = user:getpos()
@@ -403,7 +493,9 @@ end
 
 function core.item_eat(hp_change, replace_with_item)
 	return function(itemstack, user, pointed_thing)  -- closure
-		return core.do_item_eat(hp_change, replace_with_item, itemstack, user, pointed_thing)
+		if user then
+			return core.do_item_eat(hp_change, replace_with_item, itemstack, user, pointed_thing)
+		end
 	end
 end
 
@@ -420,65 +512,82 @@ end
 
 function core.handle_node_drops(pos, drops, digger)
 	-- Add dropped items to object's inventory
-	if digger:get_inventory() then
-		local _, dropped_item
-		for _, dropped_item in ipairs(drops) do
-			local left = digger:get_inventory():add_item("main", dropped_item)
-			if not left:is_empty() then
-				local p = {
-					x = pos.x + math.random()/2-0.25,
-					y = pos.y + math.random()/2-0.25,
-					z = pos.z + math.random()/2-0.25,
-				}
-				core.add_item(p, left)
-			end
+	local inv = digger and digger:get_inventory()
+	local give_item
+	if inv then
+		give_item = function(item)
+			return inv:add_item("main", item)
+		end
+	else
+		give_item = function(item)
+			-- itemstring to ItemStack for left:is_empty()
+			return ItemStack(item)
+		end
+	end
+
+	for _, dropped_item in pairs(drops) do
+		local left = give_item(dropped_item)
+		if not left:is_empty() then
+			local p = {
+				x = pos.x + math.random()/2-0.25,
+				y = pos.y + math.random()/2-0.25,
+				z = pos.z + math.random()/2-0.25,
+			}
+			core.add_item(p, left)
 		end
 	end
 end
 
 function core.node_dig(pos, node, digger)
-	local def = ItemStack({name=node.name}):get_definition()
-	if not def.diggable or (def.can_dig and not def.can_dig(pos,digger)) then
-		core.log("info", digger:get_player_name() .. " tried to dig "
+	local diggername = user_name(digger)
+	local log = make_log(diggername)
+	local def = core.registered_nodes[node.name]
+	if def and (not def.diggable or
+			(def.can_dig and not def.can_dig(pos, digger))) then
+		log("info", diggername .. " tried to dig "
 			.. node.name .. " which is not diggable "
 			.. core.pos_to_string(pos))
 		return
 	end
 
-	if core.is_protected(pos, digger:get_player_name()) and
-			not minetest.check_player_privs(digger, "protection_bypass") then
-		core.log("action", digger:get_player_name()
+	if is_protected(pos, diggername) then
+		log("action", diggername
 				.. " tried to dig " .. node.name
 				.. " at protected position "
 				.. core.pos_to_string(pos))
-		core.record_protection_violation(pos, digger:get_player_name())
+		core.record_protection_violation(pos, diggername)
 		return
 	end
 
-	core.log('action', digger:get_player_name() .. " digs "
+	log('action', diggername .. " digs "
 		.. node.name .. " at " .. core.pos_to_string(pos))
 
-	local wielded = digger:get_wielded_item()
-	local drops = core.get_node_drops(node.name, wielded:get_name())
+	local wielded = digger and digger:get_wielded_item()
+	local drops = core.get_node_drops(node, wielded and wielded:get_name())
 
-	local wdef = wielded:get_definition()
-	local tp = wielded:get_tool_capabilities()
-	local dp = core.get_dig_params(def.groups, tp)
-	if wdef and wdef.after_use then
-		wielded = wdef.after_use(wielded, digger, node, dp) or wielded
-	else
-		-- Wear out tool
-		if not core.setting_getbool("creative_mode") then
-			wielded:add_wear(dp.wear)
+	if wielded then
+		local wdef = wielded:get_definition()
+		local tp = wielded:get_tool_capabilities()
+		local dp = core.get_dig_params(def and def.groups, tp)
+		if wdef and wdef.after_use then
+			wielded = wdef.after_use(wielded, digger, node, dp) or wielded
+		else
+			-- Wear out tool
+			if not core.settings:get_bool("creative_mode") then
+				wielded:add_wear(dp.wear)
+				if wielded:get_count() == 0 and wdef.sound and wdef.sound.breaks then
+					core.sound_play(wdef.sound.breaks, {pos = pos, gain = 0.5})
+				end
+			end
 		end
+		digger:set_wielded_item(wielded)
 	end
-	digger:set_wielded_item(wielded)
 
 	-- Handle drops
 	core.handle_node_drops(pos, drops, digger)
 
 	local oldmetadata = nil
-	if def.after_dig_node then
+	if def and def.after_dig_node then
 		oldmetadata = core.get_meta(pos):to_table()
 	end
 
@@ -486,7 +595,7 @@ function core.node_dig(pos, node, digger)
 	core.remove_node(pos)
 
 	-- Run callback
-	if def.after_dig_node then
+	if def and def.after_dig_node then
 		-- Copy pos and node because callback can modify them
 		local pos_copy = {x=pos.x, y=pos.y, z=pos.z}
 		local node_copy = {name=node.name, param1=node.param1, param2=node.param2}
