@@ -28,7 +28,8 @@ local function get_formspec(tabview, name, tabdata)
 
 	local retval =
 		"label[0.05,-0.25;".. fgettext("Installed Mods:") .. "]" ..
-		"textlist[0,0.25;5.1,5;modlist;" ..
+		"tablecolumns[color;tree;text]" ..
+		"table[0,0.25;5.1,5;modlist;" ..
 		modmgr.render_modlist(modmgr.global_mods) ..
 		";" .. tabdata.selected_mod .. "]"
 
@@ -74,7 +75,7 @@ local function get_formspec(tabview, name, tabdata)
 		if error == nil then
 			local descriptiontext = descriptionfile:read("*all")
 
-			descriptionlines = core.splittext(descriptiontext,42)
+			descriptionlines = core.wrap_text(descriptiontext, 42, true)
 			descriptionfile:close()
 		else
 			descriptionlines = {}
@@ -98,12 +99,24 @@ local function get_formspec(tabview, name, tabdata)
 				.. fgettext("Uninstall selected modpack") .. "]"
 		else
 			--show dependencies
+			local toadd_hard, toadd_soft = modmgr.get_dependencies(selected_mod.path)
+			if toadd_hard == "" and toadd_soft == "" then
+				retval = retval .. "," .. fgettext("No dependencies.")
+			else
+				if toadd_hard ~= "" then
+					retval = retval .. "," .. fgettext("Dependencies:") .. ","
+					retval = retval .. toadd_hard
+				end
+				if toadd_soft ~= "" then
+					if toadd_hard ~= "" then
+						retval = retval .. ","
+					end
+					retval = retval .. "," .. fgettext("Optional dependencies:") .. ","
+					retval = retval .. toadd_soft
+				end
+			end
 
-			retval = retval .. "," .. fgettext("Depends:") .. ","
-
-			local toadd = modmgr.get_dependencies(selected_mod.path)
-
-			retval = retval .. toadd .. ";0]"
+			retval = retval .. ";0]"
 
 			retval = retval .. "button[5.5,4.85;4.5,0.5;btn_mod_mgr_delete_mod;"
 				.. fgettext("Uninstall selected mod") .. "]"
@@ -115,8 +128,8 @@ end
 --------------------------------------------------------------------------------
 local function handle_buttons(tabview, fields, tabname, tabdata)
 	if fields["modlist"] ~= nil then
-		local event = core.explode_textlist_event(fields["modlist"])
-		tabdata.selected_mod = event.index
+		local event = core.explode_table_event(fields["modlist"])
+		tabdata.selected_mod = event.row
 		return true
 	end
 
